// =============================
// Backbone setup
// =============================
var app = app || {};
var formatCharCount = window.SporcleLib.formatCharCount;

// Add a fullExtend method to views
(function(View){
	'use strict';
	// Additional extension layer for Views
	View.fullExtend = function (protoProps, staticProps) {
		// Call default extend method
		var extended = View.extend.call(this, protoProps, staticProps);
		// Add a usable super method for better inheritance
		extended.prototype._super = this.prototype;
		// Apply new or different events on top of the original
		if (protoProps.events) {
			for (var k in this.prototype.events) {
				if (!extended.prototype.events[k]) {
					extended.prototype.events[k] = this.prototype.events[k];
				}
			}
		}
		return extended;
	};

})(Backbone.View);


// =============================
// Models
// =============================
app.Models = app.Models || {};

app.Models.Comment = Backbone.Model.extend({
	url: function() {
		return _.result(this.collection, 'url')+'?comment_id='+this.get('id')+'&parent_code='+this.collection.parentCode;
	},
	defaults: {
		id              : 0,
		parent_id       : 0,
		raw_text        : '',
		text            : '',
		score           : 0,
		up_votes        : 0,
		down_votes      : 0,
		hidden          : 0,
		comment_state   : 0,
		archived        : 0,
		show_delete     : false,
		show_edit       : false,
		is_mobile       : 0,
		timestamp       : 0,
		time_display    : '',
		time_string     : '',
		user_id         : 0,
		user_handle     : '',
		user_image      : '',
		user_vote       : 0,
		user_reported   : false,
		user_is_deleted	: 0,
		collapsed       : 0,
		commenter_role  : "",
		is_creator      : false,
		active_urls     : false,
		is_edited       : 0
	},
	toJSON: function() {
		return _.extend(
			_.pick(
				this.attributes,
				'id', 'raw_text', 'text', 'score', 'up_votes', 'down_votes', 'hidden',
				'comment_state', 'archived','show_delete', 'show_edit',
				'is_mobile', 'timestamp', 'time_display', 'time_string', 'user_handle',
				'user_image', 'user_vote', 'user_reported', 'user_is_deleted',
				'commenter_role', 'is_creator', 'active_urls', 'is_edited')
		);
	}
});


// =============================
// Collections
// =============================
app.Collections = app.Collections || {};

app.Collections.Comments = Backbone.Collection.extend({
	model: app.Models.Comment,
	initialize: function(options) {
		var self = this;

		this.url = options.url;
		this.parentCode = options.parentCode;
		this.userID = options.userID;
		this.pageNum = options.pageNum;
		this.numPages = options.numPages;
		this.perPage = options.perPage;
		this.totalCount = options.totalCount;
		this.userCount = options.userCount;
		this.previousComment = options.previousComment;
		this.archiveCount = 0;
		this.sortDesc = true;
		this.sort_by = 'comment_time';
		this.allExpanded = true;

		this._showArchive = false;
		this._commentCount = 0;
		this._archiveCount = 0;

		this.listenTo(this, 'reset', function(collection, options) {
			this._updateCount('reset', collection);

			self.allExpanded = true;

			for (var i = 0; i < this.models.length; i++) {
				if (this.models[i].get('collapsed')) {
					self.allExpanded = false;
					break;
				}
			}
		}, this);

		this.listenTo(this, 'add', function(model, collection, options) {
			this._updateCount('add', model);
		});

		this.listenTo(this, 'remove', function(model, collection, options) {
			this._updateCount('remove', model);
		});
	},
	parse: function(response) {
		if (response.success) {
			this.totalCount = parseInt(response.total_count);
			this.numPages = Math.ceil(this.totalCount/this.perPage);
			this.userCount = parseInt(response.user_count);
			this.previousComment = parseInt(response.prev_comment);

			if (typeof response.page != 'undefined') {
				this.pageNum = parseInt(response.page);
			}

			if (typeof response.archiveCount != 'undefined') {
				this.archiveCount = parseInt(response.archiveCount);
			}

	        return response.comments;
	    }

	    return response;
	},
	fetchPage: function(callback) {
		var parentCode = this.parentCode;
		var pageNum = this.pageNum;
		var sortBy = this.sort_by;
		var perPage = this.perPage;
		var sortDesc = this.sortDesc;

		var self = this;

		//Update page titles for paginated comments on group/bulletin pages (not game page because its url doesn't update when paginating comments)
		if(typeof gameID == 'undefined' || !gameID){

            var twitterTitle = document.querySelector('meta[name="twitter:title"]');
            var fbTitle = document.querySelector('meta[property="og:title"]');

            //Don't want to show page number on first page's title
            if(pageNum <= 1){

                document.title = document.title.replace(/ - Page \d+/, '');
                twitterTitle.content = twitterTitle.content.replace(/ - Page \d+/, '');
                fbTitle.content = fbTitle.content.replace(/ - Page \d+/, '');

            //If we landed on first page and then went to another, need to append Page # entirely
            } else if(pageNum > 1 && document.title.indexOf('- Page') == -1){

                document.title += (' - Page ' + pageNum);
                twitterTitle.content += (' - Page ' + pageNum);
                fbTitle.content += (' - Page ' + pageNum);

            //If page number already exists in title and we're still beyond first page, just need to replace number
            } else if (pageNum > 1 && document.title.indexOf('- Page') != -1){

                document.title = document.title.replace(/Page \d+/, 'Page ' + pageNum);
                twitterTitle.content = twitterTitle.content.replace(/Page \d+/, 'Page ' + pageNum);
                fbTitle.content = fbTitle.content.replace(/Page \d+/, 'Page ' + pageNum);

            }
        }

        this.fetch({
			data: {
				parent_code : parentCode,
				page        : pageNum,
				sort_by     : sortBy,
				perPage     : perPage,
				sortDesc    : sortDesc
			},
			reset:true
		});
	},
	_updateCount: function(e, object) {
		if (e == "add" || e == "remove") {
			action = {"add":1,"remove":-1}

			this.totalCount += action[e];

			if (object.get('archived')) {
				this._archiveCount += action[e]
			} else {
				this._commentCount += action[e]
			}

			if ((this.previousComment == 0 && e == "add") || (this.previousComment == 1 && e == "remove")) {
				this.userCount += action[e];
			}
			this.previousComment += action[e];
		} else if (e == "reset") {
			var archiveCount = 0;
			var commentCount = 0;

			_.each(this.models, function(comment) {
				if (comment.get('comment_state') !== 'removed') {
					if (comment.get('archived')) {
						archiveCount++;
					} else {
						commentCount++;
					}
				}
			});

			this._commentCount = commentCount;
			this._archiveCount = archiveCount;
		}
	},
	expandAll: function() {
		this.allExpanded = true;

		// Set all comments to "new" so they expand when rendered
		_.each(this.models, function(model) {
			model.set({'collapsed' : 0}, {silent: true});
		});

		this.trigger('expand');

	},
	collapseAll: function() {
		this.allExpanded = false;

		// Set all comments to "new" so they expand when rendered
		_.each(this.models, function(model) {
			if(!(model.has('is_bulletin_body') && model.get('is_bulletin_body')))
				model.set({'collapsed' : 1}, {silent: true});
		});

		this.trigger('collapse');

	},
	toJSON: function() {
		if (this.length > 10) {
			var nextPage = true;
		} else {
			var nextPage = false;
		}

		return {
			'sortBy'              : this.sort_by,
			'sortDesc'            : this.sortDesc,
			'numPages'            : this.numPages,
			'pageNum'             : this.pageNum,
			'perPage'             : this.perPage,
			'comment_count'       : this._commentCount,
			'archive_count'       : this._archiveCount,
			'show_archive'        : this._showArchive,
			'total_comment_count' : this.totalCount+this.archiveCount,
			'unique_user_count'	  : this.userCount,
			'allExpanded'         : this.allExpanded
		};
	}
});

// =============================
// Views
// =============================
app.Views = app.Views || {};

app.Views.CommentView = Backbone.View.extend({
	tagName: 'li',
	className: 'comment',
	initialize: function() {
		this.listenTo(this.model, "change:score", this.renderVote, this);
		this.listenTo(this.model, "change:hidden", this.render, this);
		this.listenTo(this.model, "change:collapsed", this.render, this);

		this.editing = false;
	},
	events: {
		'click .show-comment'         : 'showComment',
		'click .up-vote'              : 'upVote',
		'click .down-vote'            : 'downVote',
		'click .up-vote'              : 'upVote',
		'click .down-vote'            : 'downVote',
		'click .delete-comment'       : 'deleteComment',
		'click .delete-blocked'		  : 'blockDelete',
		'click .edit-comment'         : 'editComment',
        'click .edit-blocked'         : 'blockEdit',
		'click .save-edit'            : 'saveEdit',
		'click .cancel-edit'          : 'cancelEdit',
		'click .report-comment'       : 'showReportModal',
		'click .remove-comment'       : 'removeComment',
		'click .remove-blocked'		  : 'blockRemove',
		'click .admin-delete-comment' : 'adminDeleteComment',
		'click .admin-delete-blocked' : 'blockDelete',
		'click .collapsedContent'     : 'showComment'
	},
	render: function() {
		var templateName = this.model.get('collapsed') ? '#collapsed-comment-template' : '#comment-template';
		var template = _.template($j(templateName).html());
		var data = this.model.toJSON();
		data.renderUserIcon = _.template($j('#user-thumb-template').html());
		data.renderUserLink = _.template($j('#user-anchor-template').html());
		data.text = window.SporcleLib.Sitdown.render(data.text, data.active_urls);
		data.editing = this.editing;

		this.$el.html(template(data));
		this.toggleHidden();
		return this;
	},
	toggleHidden: function() {
		//Don't ever hide or collapse bulletin description comments regardless of score
		if(this.model.has('is_bulletin_body') && this.model.get('is_bulletin_body')){
			this.el.className = 'standard comment';
		} else if (this.model.get('hidden') || (this.model.get('comment_state') === 'removed')) {
			this.el.className = 'hidden unpopular comment';
		} else if (this.model.get('delete')) {
			this.el.className = 'deleted comment';
		} else if (this.model.get('collapsed')) {
			this.el.className = 'collapsed comment';
		} else {
			this.el.className = 'standard comment';
		}
	},
	renderVote: function() {
		var user_vote = this.model.get('user_vote');
		if (user_vote == 1) {
			this.$el.find('.up-vote').removeClass("vOff").addClass('vSelect');
			this.$el.find('.down-vote').removeClass("vSelect").addClass('vOff');
		} else if (user_vote == -1) {
			this.$el.find('.up-vote').removeClass("vSelect").addClass('vOff');
			this.$el.find('.down-vote').removeClass("vOff").addClass('vSelect');
		} else if (user_vote == 0) {
			this.$el.find('.up-vote').removeClass("vSelect").removeClass('vOff');
			this.$el.find('.down-vote').removeClass("vOff").removeClass('vSelect');
		}

		var $points = this.$el.find('.points');
		var score = this.model.get('score');

		$points.html(score + ' point'+(score == 1 ? '' : 's'));

		if ($points.attr('title')) {
			$points.prop('title', this.model.get('up_votes') + ' Up, ' + this.model.get('down_votes') + ' Down');
		}

	},
	close: function() {
		// Completely unbind the view
		this.undelegateEvents();
		this.$el.removeData().off();

		// Remove view from DOM
		this.remove();
		Backbone.View.prototype.remove.call(this);

		// trigger a close event to update other modules, if any is listenting
		this.trigger('close');
	},
	showComment: function() {
		this.model.set({hidden:0, collapsed:0});
	},
	upVote: function(e) {
		this.castVote({
			vote : 1
		});
	},
	downVote: function(e) {
		this.castVote({
			vote : -1
		});
	},
	showReportModal: function() {
		var self = this;
		var reportCallback = function() {
			self.model.save({report:1}, {
				patch:true,
				success: function() {
					self.model.set({user_reported: true});
					self.render();
				},
				error: function(model, response) {
					if (response.responseJSON.hasOwnProperty('message')) {
						showMainPageAlert(response.responseJSON.message, {type:'danger'});
					} else {
						showMainPageAlert("Oops, something went wrong. Please refresh and try again.", {type:'danger'});
					}
				}
			});
		};

		SporcleLib.Modal.openReportCommentModal({
			comment_id: this.model.get('id'),
			callback: reportCallback
		});
	},
	removeComment: function() {
        var self = this;

        window.SporcleLib.Modal.openConfirmationModal({
            msg: "Are you sure you want to remove this comment?",
            onYes: function (remodal) {
                self.model.save({remove:1}, {
                    patch: true,
                    success: function() {
                        self.model.set({comment_state: 'removed'});
                        self.render();
                    },
                    error: function(model, response) {
                        if (response.responseJSON.hasOwnProperty('message')) {
                            showMainPageAlert(response.responseJSON.message, {type:'danger'});
                        } else {
                            showMainPageAlert("Oops, something went wrong. Please refresh and try again.", {type:'danger'});
                        }
                    }
                });
                remodal.close();
            }
        });
	},
	blockRemove: function() {
		var message = "Your account has been blocked from removing comments. Contact <a href='/feedback/'>customer support</a> to find out why.";
		showMainPageAlert(message, {type: 'danger'});
	},
	blockDelete: function() {
		var message = "Your account has been blocked from deleting comments. Contact <a href='/feedback/'>customer support</a> to find out why.";
		showMainPageAlert(message, {type: 'danger'});
	},
	adminDeleteComment: function() {
		var self = this;
		var msg = "Are you sure you want to delete this comment?";
		var data = {delete: 1};

		if(this.model.has('is_bulletin_body') && this.model.get('is_bulletin_body')){
			msg += " Doing so will also delete the bulletin."
			data.is_bulletin_body = 1;
		}
        window.SporcleLib.Modal.openConfirmationModal({
            msg: msg,
            onYes: function (remodal) {
                self.performDelete(data);
              	remodal.close();
            }
        });
	},
	deleteComment: function() {
        var self = this;
		var msg = "Are you sure you want to delete your comment?";
		var data = {delete:1, user_id:self.model.get('user_id')}

		if(this.model.has('is_bulletin_body') && this.model.get('is_bulletin_body')){
			msg += " Doing so will also delete the bulletin."
			data.is_bulletin_body = 1;
		}

        window.SporcleLib.Modal.openConfirmationModal({
            msg: msg,
            onYes: function (remodal) {
                self.performDelete(data);
                remodal.close();
            }
        });
	},
	editComment: function() {
		this.editing = true;
		this.render();
	},
    blockEdit: function() {
        var message = "Your account has been blocked from editing comments. Contact <a href='/feedback/'>customer support</a> to find out why.";
        showMainPageAlert(message, {type: 'danger'});
    },
	saveEdit: function() {
		var self = this;

		var newText = this.$('.comment-edit').val();
		if (newText === this.model.get('raw_text')) {
			this.cancelEdit();
			return;
		}

		var changeAttrs = {
			raw_text: newText,
			is_edited: 1
		};

		var oldAttrs = Object.keys(changeAttrs).reduce(function(previous, current) {
			previous[current] = self.model.get(current);
			return previous;
		}, {});

		this.model.save(changeAttrs, {
			patch: true,
			success: function() {
				self.editing = false;
				self.render();
			},
			error: function(model, response) {
				self.model.set(oldAttrs);
				if (response.responseJSON.hasOwnProperty('message')) {
					showMainPageAlert(response.responseJSON.message, {type:'danger'});
				} else {
					showMainPageAlert("Oops, something went wrong. Please refresh and try again.", {type:'danger'});
				}
			}
		});
	},
	cancelEdit: function() {
		this.editing = false;
		this.render();
	},
	castVote: function(options) {
		var self = this;

		var curUserVote = this.model.get('user_vote');
		var curCommentScore = this.model.get('score');

		var userVote = curUserVote + options.vote;
		var commentScore = curCommentScore + options.vote;

		var upVotes = this.model.get('up_votes');
		var downVotes = this.model.get('down_votes');

		if (curUserVote == options.vote) {
			if (options.vote == 1) {
				upVotes -= 1;
			} else if (options.vote == -1) {
				downVotes -= 1;
			}

			userVote = curUserVote + options.vote*-1;
			commentScore = curCommentScore + options.vote*-1;
			options.vote = 0;
		} else if (curUserVote == options.vote*-1) {
			userVote = curUserVote + options.vote*2;
			commentScore = curCommentScore + options.vote*2;

			if (options.vote == 1) {
				upVotes += 1;
				downVotes -= 1;
			} else if (options.vote == -1) {
				upVotes -= 1;
				downVotes += 1;
		}
		} else {
			if (options.vote == 1) {
				upVotes += 1;
			} else if (options.vote == -1) {
				downVotes += 1;
			}
		}

		var changeAttrs = {
			user_vote : userVote,
			score : commentScore,
			up_votes : upVotes,
			down_votes : downVotes
		};

		var oldAttrs = Object.keys(changeAttrs).reduce(function(previous, current) {
			previous[current] = self.model.get(current);
			return previous;
		}, {});

		this.model.save(options, {
			wait: false,
			patch: true,
            success: function() {
                self.model.set(changeAttrs);
            },
			error: function(model, response) {
				self.model.set(oldAttrs);
				if (response.responseJSON.hasOwnProperty('message')) {
					showMainPageAlert(response.responseJSON.message, {type:'danger'});
				} else {
					showMainPageAlert("Oops, something went wrong. Please refresh and try again.", {type:'danger'});
				}
			}
		});
	},
	performDelete: function(options) {
		var self = this;
		this.model.save(options, {
			patch: true,
			success: function(response, data) {
				if (response.attributes.redirect && response.attributes.redirect_location) {
					window.location = response.attributes.redirect_location;
					return;
				}
				
				self.model.trigger('destroy', self.model);
				self.close();
			},
			error: function(model, response) {
				if (response.responseJSON.hasOwnProperty('message')) {
					showMainPageAlert(response.responseJSON.message, {type:'danger'});
				} else {
					showMainPageAlert("Oops, something went wrong. Please refresh and try again.", {type:'danger'});
				}
			}
		});
	}
});

app.Views.CommentsView = Backbone.View.extend({
	el: '#comments-section',
	template: _.template($j('#comment-section-template').html()),
	initialize: function(options, groupCode) {

		var self = this;

		_.defaults(options, {
			'topic_locked' : false,
			'pushState'    : false
		});

		this.pushState = options.pushState;
		this._topicLocked = options.topic_locked;
		this._numUrlParts = options.num_base_url_parts ? options.num_base_url_parts : 4;
		this._basePathName = window.location.pathname.split("/").slice(0,this._numUrlParts).join("/");
		this.noPush = true;

		this.commentHelp = new app.Views.CommentHelpView();

		// Used to keep track of subviews:
		this.commentViews = [];

		// Used for user name suggestions
		this.commenters = [];
		if (options.creatorHandle) {
			this.commenters.push(options.creatorHandle);
		}

		this.showingSuggestions = false;

		this.listenTo(this.collection, 'remove', this.removeComment, this);
		this.listenTo(this.collection, 'add', this.addNewComment, this);
		this.listenTo(this.collection, 'reset sort expand collapse', this.render, this);

		if (this.pushState) {
			window.onpopstate = function (event) {
				if (event.state) {
					self.noPush = true;
					self.collection.pageNum = event.state.pageNum;
					self._goPage();
				}
			}
		}
	},
	events: {
		'click #toggle-archive'             : 'toggleArchive',
		'click #submit-comment'				: 'newComment',
		'click #firstPage.activeLink' : 'firstPage',
		'click #prevPage.activeLink'        : 'prevPage',
		'click #nextPage.activeLink'        : 'nextPage',
		'click #lastPage.activeLink'        : 'lastPage',
		'click .pagenum:not(.active)'       : 'goPageNum',
		'click #sort-by-time-new'     : 'sortByNewest',
		'click #sort-by-time-old'     : 'sortByOldest',
		'click #sort-by-score-top'    : 'sortByTopScore',
		'click #sort-by-score-low'    : 'sortByLowScore',
		'click #expand-all'           : 'expandAll',
		'click .sign-up-light-box'          : 'showLightBox',
		'click .unverified-light-box'       : 'showUnverifiedLightBox',
		'click #commentHelp'                : 'showCommentHelp',
		'click #comment_textarea'			: 'suggestUserNames',
		'keyup #comment_textarea'			: 'processTextInput',
		'keydown #comment_textarea'			: 'navigateSuggestions'
	},
	addNewComment: function(comment) {

		var oldSort = this.collection.sort_by;
		var oldDesc = this.collection.sortDesc;

		this.collection.sort_by = 'comment_time';
		this.collection.sortDesc = true;

		if ((this.collection.numPages > 0) && (this.collection.pageNum != this.collection.numPages)) {
			this.scrollToComment = true;
			this.collection.pageNum = this.collection.numPages;
			this._goPage();
		} else if ((((this.collection.models.length) > this.collection.perPage))) {
			this.scrollToComment = true;
			this.collection.numPages = this.collection.numPages + 1;
			this.collection.pageNum = this.collection.numPages;
			this._goPage();
		} else if ((oldSort != 'comment_time') || (!oldDesc)) {
			var collection = this.collection;

			if (this.collection.sortDesc) {
				this.collection.comparator = function (comment) {
					return comment.get("timestamp");
				};
			} else {
				this.collection.comparator = function (comment) {
					return -comment.get("timestamp");
				};
			}

			this._goPage();
		} else {
			this._renderOne(comment);
			this.updateCount();

		}
	},
	removeComment: function(comment) {
		this.collection.sort_by = 'comment_time';
		this.collection.sortDesc = true;

		if (this.collection.models.length < 1) {
			this.collection.numPages = Math.max(1, this.collection.numPages - 1);
			this.collection.pageNum = this.collection.numPages;
			this._goPage();
		} else {
			this.updateCount();
		}
	},
	_renderOne: function(comment) {
		var newComment = new app.Views.CommentView({model: comment});
		this.commentViews.push(newComment);
		var commenter = comment.get('user_handle');
		if (this.commenters.indexOf(commenter) === -1 && !comment.get('user_is_deleted')) {
			this.commenters.push(commenter);
		}

		this.commentHelp.listenTo(newComment, 'close', this.commentHelp.reposition);

		if (comment.get('archived')) {
			this.archiveList.append(newComment.render().$el);
			this.archiveCount++;
		} else {
			this.commentList.append(newComment.render().$el);
			this.commentCount++;
		}
	},
	render: function() {
		if (this.pushState && !this.noPush) {
			address = this.collection.pageNum === 1 ? this._basePathName : this._basePathName+"/"+this.collection.pageNum;
			window.history.pushState({pageNum:this.collection.pageNum}, "", address);
		} else {
			this.noPush = false;
		}

		this.$el.html(this.template(this.collection.toJSON()));

		//Make sure we have a clean slate in terms of post models:
		if (this.commentViews.length > 0) {
			_.each(this.postViews, function(view) {
				view.close();
			});
			this.commentViews = [];
		}

		this.archiveList = this.$el.find('#comments-archive');
		this.commentList = this.$el.find('#comments-current');
		this.commentCount = 0;
		this.archiveCount = 0;

		var self = this;
		_.each(this.collection.models, function(comment) {
			self._renderOne(comment);
		});

		if (this._topicLocked) {
			this.$el.find('#new-comment-box').html('<div id="groupSuccess" class="alertBad" style="display: block;">This topic has been locked by the Sporcle admin.</div>');
		} else {
			this.$el.find('#new-comment-box').html($j('#new-comment-box-template').html());
			this.$commentTextArea = this.$el.find('#comment_textarea');
			this.$el.find('#char-count').html(formatCharCount(0, this.$commentTextArea.prop('maxlength')));
			this.$suggestionsWrapper = this.$el.find('#handle-suggestions');
			this.$suggestions = this.$suggestionsWrapper.find('#suggestions');
			this.commenters.sort(function(a,b) {
				return a.localeCompare(b, 'en', {'sensitivity': 'base'});
			});
		}

		this.collection._archiveCount = this.archiveCount;

		this.repositionCommentHelp();

		this.$el.css({ opacity: 1});
		this.$el.css('pointer-events', '');

		if (this.scrollToComment) {
			this.scrollToComment = false;
			$j('html, body').scrollTop(this.$el.find('#new-comment-box').offset().top-250);
		} else if (this.scrollToTop) {
			this.scrollToTop = false;
			$j('html, body').scrollTop(self.$el.offset().top-10);
		}

		return this;
	},
	toggleArchive: function() {
		this.collection._showArchive = !this.collection._showArchive;
		var archive = this.$el.find('#archive-box');
		archive.toggle();
		this.$el.find('#toggle-label').html(archive.is(':visible') ? 'Hide' : 'Show');
		this.repositionCommentHelp();
	},
	newComment: function() {
        //If the user is not logged in or unverified, abort the function. We'll show a log in or verify modal instead. USER_STATUS is passed in from show_comments.php
		if(!USER_STATUS.isLoggedIn) {
			return this.showLightBox();
		} else if (!USER_STATUS.isVerified) {
			return this.showUnverifiedLightBox();
		}

        var self = this;
        var userComment = this.$commentTextArea.val();

        this.closeCommentHelp();

		var $newCommentBox = this.$el.find('#new-comment-box');

		$newCommentBox.css({ opacity: 0.25});
		$newCommentBox.css('pointer-events', 'none');

		this.collection.create({raw_text: userComment, text: userComment, score:1, user_vote:1}, {
			wait: true,
			success: function(model, response, options) {
				self.collection.add(model);
				self.$commentTextArea.val('');

				$newCommentBox.css({ opacity: 1});
				$newCommentBox.css('pointer-events', '');
				self.$el.find('#char-count').html(formatCharCount(0, self.$commentTextArea.prop('maxlength')));
			},
			error: function(model, response) {
				self.collection.remove(model);

				$newCommentBox.css({ opacity: 1});
				$newCommentBox.css('pointer-events', '');

				if (response.responseJSON.hasOwnProperty('message')) {
					showMainPageAlert(response.responseJSON.message, {type:'danger'});
				} else {
					showMainPageAlert("Oops, something went wrong. Please refresh and try again.", {type:'danger'});
				}
			}
		});
	},
	// Get the cursor position in the given DOM object
	getCursorPos: function(el) {
		var pos = 0;
		if ('selectionStart' in el) {
			pos = el.selectionStart;
		}
		return pos;
	},
	countChars: function(target){
		var chars = target.value.length;
		var max = target.attributes.maxlength.value;
		$j('#char-count').html(formatCharCount(chars, max));
	},
	processTextInput: function(e){
		this.suggestUserNames(e);
		this.countChars(e.target);
	},
	suggestUserNames: function(e) {
		// If they are using navigating keys and the suggestion box is already open, exit out of here.
		var keyCode = e.keyCode ? e.keyCode : e.which;
		if (this.showingSuggestions && (keyCode === 13 || keyCode === 38 || keyCode === 40)) {
			return;
		}

		// We want to see if the word under the current cursor matches part of a commenter on the current page.
		var pos = this.getCursorPos(this.$commentTextArea.get(0));
		var text = this.$commentTextArea.val();

		if (pos === 0 || text.length === 0) {
			this.clearSuggestions();
			return;
		}

		var preSubstring = '', postSubstring = '';

		// Get the text to the left of the cursor, matching the format @ + allowable username characters.
		var preMatches = text.substring(0, pos).match(/@([A-Za-z0-9_\-]*)$/);
		if (preMatches != null && typeof preMatches === 'object' && preMatches.length > 1) {
			preSubstring = preMatches[1];

			// If we got a match, also get the text to the right of the cursor up to the first non-allowable username
			// character.
			var postMatches = text.substring(pos).match(/^[A-Za-z0-9_\-]*/);
			if (postMatches != null && typeof postMatches === 'object' && postMatches.length > 0) {
				postSubstring = postMatches[0];
			}
		}

		var handleSubstring = preSubstring + postSubstring;

		if (handleSubstring.length > 0) {
			// Figure out where the end of the substring we're going to use for suggestions is, which could be to the right
			// of the current cursor.
			var endOfSubstringPos = pos + postSubstring.length;

			this.showSuggestions(handleSubstring, endOfSubstringPos);

		} else {
			this.clearSuggestions();
		}
	},
	showSuggestions: function(subStr, endPos) {
		var self = this;
		this.showingSuggestions = false;
		this.$suggestions.empty();
		var count = 0;

		_.some(this.commenters, _.bind(function(userName) {
			if (userName.toLowerCase().indexOf(subStr.toLowerCase()) === 0 && userName.toLowerCase() !== subStr.toLowerCase()) {

				this.showingSuggestions = true;

				var $div = $j("<div>").html(userName).addClass('suggestion').on('click suggestion:select', function(e) {
					self.replaceWithSuggestion(subStr, userName, endPos, self);
					self.clearSuggestions();
				});
				if (this.$suggestions.is(":empty")) {
					$div.addClass('selected');
				}
				this.$suggestions.append($div);

				count++;
				return (count >= 4); // returning true breaks out of the `some` loop
			}
		}, this));

		if (this.showingSuggestions) {
			this.$suggestionsWrapper.show();
		} else {
			this.clearSuggestions();
		}
	},
	clearSuggestions: function() {
		this.showingSuggestions = false;
		this.$suggestions.empty();
		this.$suggestionsWrapper.hide();
	},
	navigateSuggestions: function(e) {
		var keyCode = e.keyCode ? e.keyCode : e.which;
		if (this.showingSuggestions && (keyCode === 13 || keyCode === 38 || keyCode === 40)) {
			e.preventDefault();

			var $selected = this.$suggestions.find('.suggestion.selected');

			if (keyCode === 38) {
				// Up arrow
				if ($selected.length) {
					if (!$selected.is(":first-child")) {
						$selected.removeClass('selected').prev().addClass('selected');
					}
				} else {
					this.$suggestions.find('.suggestion').filter(':last').addClass('selected');
				}

			} else if (keyCode === 40) {
				// Down arrow
				if ($selected.length) {
					if (!$selected.is(":last-child")) {
						$selected.removeClass('selected').next().addClass('selected');
					}
				} else {
					this.$suggestions.find('.suggestion').filter(':first').addClass('selected');
				}

			} else if (keyCode === 13) {
				// Enter key
				if ($selected.length) {
					$selected.trigger('suggestion:select');
				}
			}
		}
	},
	replaceWithSuggestion: function(subStr, suggestion, pos, self) {
		var commentText = self.$commentTextArea.val();
		var newPos = pos + (suggestion.length - subStr.length) + 1;
		var newCommentText = commentText.substr(0, pos - subStr.length) + suggestion + ' ' + commentText.substr(pos);

		self.$commentTextArea.val(newCommentText).trigger('focus').get(0).setSelectionRange(newPos, newPos);
	},
	firstPage: function() {
		this.collection.pageNum = 1;
		this._goPage();
	},
	prevPage: function() {
		this.collection.pageNum--;
		this._goPage();
	},
	nextPage: function() {
		this.collection.pageNum++;
		this._goPage();
	},
	lastPage: function() {
		this.collection.pageNum = this.collection.numPages;
		this._goPage();
	},
	_goPage: function() {
		var self = this;
		self.scrollToTop = true;
		self.collection._showArchive = false;
		this.$el.css({ opacity: 0.25 });
		this.$el.css('pointer-events', 'none');
		this.collection.fetchPage();
	},
	goPageNum: function(ev) {
		this.closeCommentHelp();
		this.collection.pageNum = parseInt($j(ev.target).text());
		this._goPage();
	},
	sortByOldest: function() {
		this.sortByTime(true);
	},
	sortByNewest: function() {
		this.sortByTime(false);
	},
	sortByTopScore: function() {
		this.sortByScore(true);
	},
	sortByLowScore: function() {
		this.sortByScore(false);
	},
	sortByTime: function(sortDesc) {
		this.collection.sortDesc = sortDesc;
		this.collection.sort_by = 'comment_time';

		if (this.collection.numPages > 1) {
			this.collection.pageNum = 1;
			this._goPage();
		} else {
			this.collection.expandAll();

			if (sortDesc) {
				this.collection.comparator = function (comment) {
					return comment.get("timestamp");
				};
			} else {
				this.collection.comparator = function (comment) {
					return -comment.get("timestamp");
				};
			}

			this.collection.sort();
		}
	},
	sortByScore: function(sortDesc) {
		this.collection.sortDesc = sortDesc;
		this.collection.sort_by = 'score';

		if (this.collection.numPages > 1) {
			this.collection.pageNum = 1;
			this._goPage();
		} else {
			this.collection.expandAll();

			if (sortDesc) {
				this.collection.comparator = function (comment) {
					return -comment.get("score");
				};
			} else {
				this.collection.comparator = function (comment) {
					return comment.get("score");
				};
			}

			this.collection.sort();
		}
	},
	expandAll: function() {
		if (this.collection.allExpanded) {
			this.collection.collapseAll();
		} else {
			this.collection.expandAll();
		}
	},
	updateCount: function() {
		var totalCount = this.collection.totalCount + this.collection.archiveCount;
		var userCount = this.collection.userCount;
		$j('#comment-number').html(formatNumber(totalCount)+" Comment"+(totalCount == 1 ? "" : "s"));
		$j('#user-number').html(" from "+formatNumber(userCount)+" unique user"+(userCount == 1 ? "" : "s"));
	},
	showLightBox: function(e) {
		window.SporcleLib.Modal.openRegisterModal({
			analyticAction : 'Comments',
			regPath		   : 'leave_comment'
		});
	},
	showUnverifiedLightBox: function(e) {
		window.SporcleLib.Modal.openUnverifiedModal({
			promoActionText : 'Commenting'
		});
	},
	showCommentHelp: function () {
		this.commentHelp.open();
		return this;
	},
	closeCommentHelp: function () {
		this.commentHelp.close();
		return this;
	},
	repositionCommentHelp: function () {
		this.commentHelp.reposition();
		return this;
	},
	close: function() {
		// Clear out the subviews first
		_.each(this.commentViews, function(view) {
			view.close();
		});
		this.commentViews = [];

		// Completely unbind the view
		this.undelegateEvents();
		this.$el.removeData().off();

		// Remove view from DOM
		this.remove();
		Backbone.View.prototype.remove.call(this);
	}
});

app.Views.GameCommentsView = app.Views.CommentsView.fullExtend({
	events: {
		"click #hideComments"                : "hideComments",
		"click #showComments:not(.disabled)" : "showComments"
	},
	initialize: function(options) {
		this.commentViews = new Array();
		this.options = options || {};
		if (this.options.gid) {
			_.bindAll(this, 'render', 'toggleArchive');

			var self = this;
			var loading = $j('#commentLoading');
			loading.show();

			this.collection = new app.Collections.Comments({
				url        : '/games/includes/comments.php',
				parentCode : this.options.gid,
				pageNum    : 1,
				perPage    : 150,
			});

            this._super.initialize.call(this, options);

			this.collection.fetchPage();
		}
	},
	hideComments: function() {

		this.closeCommentHelp();

		$j('#commentBox').hide();
		$j('#hideComments').hide();
		$j('#commentPaginate').hide();
		$j('#showComments').show();
	},
	showComments: function() {
		$j('#commentBox').show();
		$j('#hideComments').show();
		$j('#commentPaginate').show();
		$j('#showComments').hide();
	},
	disableShowButton: function() {
		this.hideComments();
		$j('#showComments').addClass('disabled')
			.parent().addClass('disabled');
	},
	enableShowButton: function() {
		$j('#showComments').removeClass('disabled')
			.parent().removeClass('disabled');
	}
});

app.Views.CommentHelpView = Backbone.View.extend({
	tagName: 'div',
	className: 'comment_help',
	events: {
		'click #commentHelp-close': 'close'
	},
	initialize: function () {

		this.model = new Backbone.Model({ visible: 0 });

		this.$el.html(_.template($j('#comment-help-template').html())(this.model.toJSON()));

		this.listenTo(this.model, "change:visible", this.render);
	},
	render: function () {

		this.$anchorX = $j('#commentHelp');
		this.$anchorY = $j('#comments-section');

		if (this.model.get('visible')) {

			this.$anchorY.append(this.$el);
			this.reposition();
		} else {

			this.$el.detach();
		}

		return this.$el;
	},
	open: function () {

		this.model.set('visible', 1);

		return this;
	},
	close: function () {

		this.model.set('visible', 0);

		return this;
	},
	reposition: function () {

		if (this.model.get('visible')) {

			if (this.$el.outerHeight() > this.$anchorY.outerHeight()) {

				this.$el.css({ top: 0, bottom: 'auto' });
			} else {

				this.$el.css({ top: 'auto', bottom: 0 });
			}

			this.$el.css('right', this.$anchorX.outerWidth() + 10);
		}

		return this;
	}
});


// =============================
// End of Backbone setup
// =============================

function showError(message, delay) {
	message = typeof message !== 'undefined' ? message : "An unknown error occured.";
	delay = typeof delay !== 'undefined' ? delay : 3000;
	var err = $j('#actionError');
	err.html(message);
	err.fadeIn(400, function() {
		err.fadeOut(400);
	}).delay(delay);
}
